"""
AWS Disclaimer.

(c) 2018 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
This AWS Content is provided subject to the terms of the AWS Customer
Agreement available at https://aws.amazon.com/agreement/ or other written
agreement between Customer and Amazon Web Services, Inc.

Security Control: ElastiCache - Encryption at rest
Description:  Checks for encryption at rest being enabled on ElastiCache (Redis)
Services: ElastiCache, Cloudwatch, Lambda

Runtime: Python 3.6
Last Modified: 06/15/2018
"""


import logging
import os
import json
import boto3
from botocore.exceptions import ClientError


OUTBOUND_TOPIC_ARN = os.environ["outbound_topic_arn"]


def lambda_handler(event, context):
    """
    Main Function.

    Enables Logging
    Checks to see if atRestEncryptionEnabled exists in the event
    If so, it calls the get_at_rest_encryption_status function and sends
        a message if a Violation is found
    If not, the at-rest encryption feature is not supported
    """
    setup_logging()
    LOG.info(json.dumps(event))
    LOG.info(context)
    if 'atRestEncryptionEnabled' in event['detail']['responseElements']:
        at_rest_encryption_status = get_at_rest_encryption_status(event)
        if "Violation" in [at_rest_encryption_status]:
            message = create_non_compliance_message(
                event, at_rest_encryption_status)
            subject = "Violation - Amazon ElastiCache is out of compliance!"
            send_violation(message, subject, event, context)
        else:
            LOG.info("No violations found!")
    else:
        LOG.info("This engine version does not support the at-rest encryption feature.")


def get_at_rest_encryption_status(event):
    """
    Check at-rest encryption status.

    Checks the boolean of the atRestEncryptionEnabled key
    If it is True, sets the at_rest_encryption_status variable to a blank
    If it is False, sets the at_rest_encryption_status variable to "Violation"
    Then it returns the at_rest_encryption_status variable to the handler
    """
    LOG.info("Starting get_at_rest_encryption_status")
    if event['detail']['responseElements']['atRestEncryptionEnabled']:
        at_rest_encryption_status = ""
        LOG.info("At-rest encryption is enabled!")
    else:
        at_rest_encryption_status = "Violation"
        LOG.info(at_rest_encryption_status)
    return at_rest_encryption_status


def create_non_compliance_message(event, at_rest_encryption_status):
    """
    Build A Message.

    Creates a message with a first line stating what this violation is checking
    Creates more lines with identifying information about which specific violations were checked
    This is based upon whether or not the specific violation triggered a "Violation" variable
    Then it adds general information regarding the violation
    """
    message = ""
    message += "Amazon ElastiCache for Redis - Compliance Check" + "\n\n"
    if at_rest_encryption_status == "Violation":
        message += "Violation - At-rest encryption is not enabled! \n"
        message += "Replication Group ID: " + \
            event['detail']['responseElements']['replicationGroupId'] + "\n\n"
    message += 'Created By: ' + \
        event['detail']['userIdentity']['arn'] + "\n"
    return message


def send_violation(message, subject, event, context):
    """
    Send Violation.

    Appends additional information to the message from the Lambda Context
    Sends the message created using an API call to Amazon SNS
    """
    find_sns_region = OUTBOUND_TOPIC_ARN.split(":")
    sns_region = find_sns_region[3]
    message += '\nAccount: ' + event["account"] + "\n"
    message += "Region: " + event["detail"]["awsRegion"] + "\n"
    message += "\n\n"
    message += "This notification was generated by the Lambda function " + \
        context.invoked_function_arn
    send_client = boto3.client('sns', region_name=sns_region)
    try:
        send_client.publish(
            TopicArn=OUTBOUND_TOPIC_ARN,
            Message=message,
            Subject=subject
        )
    except ClientError as err:
        LOG.error(err)
        return False

def setup_logging():
    """
    Logging Function.

    Creates a global log object and sets its level.
    """
    global LOG
    LOG = logging.getLogger()
    log_levels = {'INFO': 20, 'WARNING': 30, 'ERROR': 40}

    if 'logging_level' in os.environ:
        log_level = os.environ['logging_level'].upper()
        if log_level in log_levels:
            LOG.setLevel(log_levels[log_level])
        else:
            LOG.setLevel(log_levels['ERROR'])
            LOG.error("The logging_level environment variable is not set to INFO, WARNING, or \
                      ERROR.  The log level is set to ERROR")
    else:
        LOG.setLevel(log_levels['ERROR'])
        LOG.warning('The logging_level environment variable is not set. The log level is set to \
                    ERROR')
        LOG.info('Logging setup complete - set to log level ' + str(LOG.getEffectiveLevel()))
